@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,7 +26,7 @@ global_copyright = """\
 // Generated code for Python source for module '%(name)s'
 // created by Nuitka version %(version)s
 
-// This code is in part copyright 2013 Kay Hayen.
+// This code is in part copyright 2014 Kay Hayen.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -44,6 +44,9 @@ global_copyright = """\
 template_metapath_loader_compiled_module_entry = """\
 { (char *)"%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ), NUITKA_COMPILED_MODULE },"""
 
+template_metapath_loader_compiled_package_entry = """\
+{ (char *)"%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ), NUITKA_COMPILED_PACKAGE },"""
+
 template_metapath_loader_shlib_module_entry = """\
 { (char *)"%(module_name)s", NULL, NUITKA_SHLIB_MODULE },"""
 
@@ -53,6 +56,11 @@ main_program = """\
 // calls the initialization code of the __main__ module.
 
 #include "structseq.h"
+
+#if %(python_no_warnings)d
+extern PyObject *const_str_plain_ignore;
+#endif
+
 #ifdef _NUITKA_WINMAIN_ENTRY_POINT
 int __stdcall WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, char* lpCmdLine, int nCmdShow )
 {
@@ -112,6 +120,10 @@ int main( int argc, char *argv[] )
     // Set the command line parameters for run time usage.
     setCommandLineParameters( argc, argv, false );
 
+#ifdef _NUITKA_STANDALONE
+    restoreStandaloneEnvironment();
+#endif
+
     // Initialize the constant values used.
     _initBuiltinModule();
     _initConstants();
@@ -159,10 +171,34 @@ int main( int argc, char *argv[] )
         assert ( _Py_Ticker >= 20 );
     }
 
-#if _NUITKA_STANDALONE
+#ifdef _NUITKA_STANDALONE
     setEarlyFrozenModulesFileAttribute();
 #endif
 
+    // Disable Python warnings if requested to.
+#if %(python_no_warnings)d
+    // Should be same as:
+    //   warnings.simplefilter("ignore", UserWarning)
+    //   warnings.simplefilter("ignore", DeprecationWarning)
+    // There is no C-API to control warnings. We don't care if it actually
+    // works, i.e. return code of "simplefilter" function is not checked.
+    {
+        PyObject *warnings = PyImport_ImportModule( "warnings" );
+        if ( warnings != NULL )
+        {
+            PyObject *simplefilter = PyObject_GetAttrString( warnings, "simplefilter" );
+
+            if ( simplefilter != NULL )
+            {
+                PyObject *result1 = PyObject_CallFunctionObjArgs( simplefilter, const_str_plain_ignore, PyExc_UserWarning, NULL );
+                Py_XDECREF( result1 );
+                PyObject *result2 = PyObject_CallFunctionObjArgs( simplefilter, const_str_plain_ignore, PyExc_DeprecationWarning, NULL );
+                Py_XDECREF( result2 );
+            }
+        }
+    }
+#endif
+
     // Execute the "__main__" module init function.
     MOD_INIT_NAME( __main__ )();
 
@@ -181,24 +217,9 @@ int main( int argc, char *argv[] )
 }
 """
 
-module_header_template = """\
-
-#include <nuitka/helpers.hpp>
-
-MOD_INIT_DECL( %(module_identifier)s );
-
-extern PyObject *module_%(module_identifier)s;
-extern PyDictObject *moduledict_%(module_identifier)s;
-
-// Declarations from this module to other modules if any.
-%(extra_declarations)s
-"""
-
 module_body_template = """
 #include "nuitka/prelude.hpp"
 
-#include "__modules.hpp"
-#include "__constants.hpp"
 #include "__helpers.hpp"
 
 // The _module_%(module_identifier)s is a Python object pointer of module type.
@@ -210,80 +231,20 @@ module_body_template = """
 PyObject *module_%(module_identifier)s;
 PyDictObject *moduledict_%(module_identifier)s;
 
-NUITKA_MAY_BE_UNUSED static PyObject *GET_MODULE_VALUE0( PyObject *var_name )
+// The module constants used
+%(constant_decl_codes)s
+
+static void _initModuleConstants(void)
 {
-    // For module variable values, need to lookup in module dictionary or in
-    // built-in dictionary.
-
-    PyObject *result = GET_STRING_DICT_VALUE( moduledict_%(module_identifier)s, (Nuitka_StringObject *)var_name );
-
-    if (likely( result != NULL ))
-    {
-        assertObject( result );
-
-        return result;
-    }
-
-    result = GET_STRING_DICT_VALUE( dict_builtin, (Nuitka_StringObject *)var_name );
-
-    if (likely( result != NULL ))
-    {
-        assertObject( result );
-
-        return result;
-    }
-
-    PyErr_Format( PyExc_NameError, "global name '%%s' is not defined", Nuitka_String_AsString(var_name ));
-    throw PythonException();
+%(constant_init_codes)s
 }
 
-NUITKA_MAY_BE_UNUSED static PyObject *GET_MODULE_VALUE1( PyObject *var_name )
+// The module code objects.
+%(module_code_objects_decl)s
+
+static void _initModuleCodeObjects(void)
 {
-    return INCREASE_REFCOUNT( GET_MODULE_VALUE0( var_name ) );
-}
-
-NUITKA_MAY_BE_UNUSED void static DEL_MODULE_VALUE( PyObject *var_name, bool tolerant )
-{
-    int status = PyDict_DelItem( (PyObject *)moduledict_%(module_identifier)s, var_name );
-
-    if (unlikely( status == -1 && tolerant == false ))
-    {
-        PyErr_Format(
-            PyExc_NameError,
-            "global name '%%s' is not defined",
-            Nuitka_String_AsString( var_name )
-        );
-
-        throw PythonException();
-    }
-}
-
-NUITKA_MAY_BE_UNUSED static PyObject *GET_LOCALS_OR_MODULE_VALUE0( PyObject *locals_dict, PyObject *var_name )
-{
-    PyObject *result = PyDict_GetItem( locals_dict, var_name );
-
-    if ( result != NULL )
-    {
-        return result;
-    }
-    else
-    {
-        return GET_MODULE_VALUE0( var_name );
-    }
-}
-
-NUITKA_MAY_BE_UNUSED static PyObject *GET_LOCALS_OR_MODULE_VALUE1( PyObject *locals_dict, PyObject *var_name )
-{
-    PyObject *result = PyDict_GetItem( locals_dict, var_name );
-
-    if ( result != NULL )
-    {
-        return INCREASE_REFCOUNT( result );
-    }
-    else
-    {
-        return GET_MODULE_VALUE1( var_name );
-    }
+%(module_code_objects_init)s
 }
 
 // The module function declarations.
@@ -292,6 +253,7 @@ NUITKA_MAY_BE_UNUSED static PyObject *GET_LOCALS_OR_MODULE_VALUE1( PyObject *loc
 // The module function definitions.
 %(module_functions_code)s
 
+
 #if PYTHON_VERSION >= 300
 static struct PyModuleDef mdef_%(module_identifier)s =
 {
@@ -315,6 +277,7 @@ static struct PyModuleDef mdef_%(module_identifier)s =
 
 // Table for lookup to find "frozen" modules or DLLs, i.e. the ones included in
 // or along this binary.
+%(metapath_module_decls)s
 static struct Nuitka_MetaPathBasedLoaderEntry meta_path_loader_entries[] =
 {
 %(metapath_loader_inittab)s
@@ -371,6 +334,9 @@ MOD_INIT_DECL( %(module_identifier)s )
     registerMetaPathBasedUnfreezer( meta_path_loader_entries );
 #endif
 
+    _initModuleConstants();
+    _initModuleCodeObjects();
+
     // puts( "in init%(module_identifier)s" );
 
     // Create the module object first. There are no methods initially, all are
@@ -415,15 +381,11 @@ MOD_INIT_DECL( %(module_identifier)s )
 
     if ( PyDict_GetItem( module_dict, const_str_plain___builtins__ ) == NULL )
     {
-        PyObject *value = ( PyObject *)module_builtin;
+        PyObject *value = (PyObject *)builtin_module;
 
-#ifdef _NUITKA_EXE
-        if ( module_%(module_identifier)s != module___main__ )
-        {
-#endif
-            value = PyModule_GetDict( value );
-#ifdef _NUITKA_EXE
-        }
+        // Check if main module, not a dict then.
+#if !defined(_NUITKA_EXE) || !%(is_main_module)s
+        value = PyModule_GetDict( value );
 #endif
 
 #ifndef __NUITKA_NO_ASSERT__
@@ -445,13 +407,22 @@ MOD_INIT_DECL( %(module_identifier)s )
     // Temp variables if any
 %(temps_decl)s
 
-    // Module code
+    // Module code.
 %(module_code)s
 
-   return MOD_RETURN_VALUE( module_%(module_identifier)s );
-}
+    return MOD_RETURN_VALUE( module_%(module_identifier)s );
+%(module_exit)s
 """
 
+template_module_exception_exit = """\
+module_exception_exit:
+    PyErr_Restore( exception_type, exception_value, (PyObject *)exception_tb );
+    return MOD_RETURN_VALUE( NULL );
+}"""
+
+template_module_noexception_exit = """\
+}"""
+
 template_helper_impl_decl = """\
 // This file contains helper functions that are automatically created from
 // templates.
@@ -474,7 +445,12 @@ template_frozen_modules = """\
 #include <Python.h>
 
 // Blob from which modules are unstreamed.
+#if defined(_WIN32) && defined(_NUITKA_EXE)
+extern const unsigned char* constant_bin;
+#else
 extern "C" const unsigned char constant_bin[];
+#endif
+
 #define stream_data constant_bin
 
 // These modules should be loaded as bytecode. They must e.g. be loadable
@@ -482,9 +458,13 @@ extern "C" const unsigned char constant_bin[];
 // in this un-optimized form. These are not compiled by Nuitka, and therefore
 // are not accelerated at all, merely bundled with the binary or module, so
 // that Python library can start out.
-struct _frozen Embedded_FrozenModules[] =
+
+void copyFrozenModulesTo(void* destination)
 {
-%(frozen_modules)s
-    { NULL, NULL, 0 }
-};
+    _frozen frozen_modules[] = {
+        %(frozen_modules)s
+        { NULL, NULL, 0 }
+    };
+    memcpy(destination, frozen_modules, ( _NUITKA_FROZEN + 1 ) * sizeof( struct _frozen ));
+}
 """
