@@ -34,7 +34,10 @@ class FinalizeClosureTaking(FinalizationVisitorBase):
         # print node, node.provider
 
         for variable in node.getClosureVariables():
-            referenced = variable.getReferenced()
+
+            referenced = variable
+            while referenced.isClosureReference():
+                referenced = referenced.getReferenced()
             referenced_owner = referenced.getOwner()
 
             assert not referenced.isModuleVariable()
@@ -44,6 +47,9 @@ class FinalizeClosureTaking(FinalizationVisitorBase):
             while current is not referenced_owner:
                 if current.isExpressionFunctionBody():
                     for current_variable in current.getClosureVariables():
+                        while current_variable.getReferenced().isClosureReference():
+                            current_variable = current_variable.getReferenced()
+
                         if current_variable.getReferenced() is referenced:
                             break
                     else:
