@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,6 +16,7 @@
 #     limitations under the License.
 #
 
+from __future__ import print_function
 
 def testAssert1():
     assert False
@@ -33,22 +34,34 @@ def testAssert3():
     return 1
 
 try:
-    print "Function that will assert."
+    print("Function that will assert.")
     testAssert1()
-    print "No exception."
-except Exception, e:
-    print "Raised", type(e), e
+    print("No exception.")
+except Exception as e:
+    print("Raised", type(e), e)
 
 try:
-    print "Function that will not assert."
+    print("Function that will not assert.")
     testAssert2()
-    print "No exception."
-except Exception, e:
-    print "Raised", type(e), e
+    print("No exception.")
+except Exception as e:
+    print("Raised", type(e), e)
 
 try:
-    print "Function that will assert with argument."
+    print("Function that will assert with argument.")
     testAssert3()
-    print "No exception."
-except Exception, e:
-    print "Raised", type(e), e
+    print("No exception.")
+except Exception as e:
+    print("Raised", type(e), e)
+
+try:
+    print("Assertion with tuple argument.", end = "")
+    assert False, (3,)
+except AssertionError as e:
+    print(str(e))
+
+try:
+    print("Assertion with plain argument.", end = "")
+    assert False, 3
+except AssertionError as e:
+    print(str(e))
